package app.revanced.patches.ad

import app.revanced.extensions.injectHideCall
import app.revanced.patcher.PatcherData
import app.revanced.patcher.extensions.or
import app.revanced.patcher.patch.*
import app.revanced.patcher.signature.MethodMetadata
import app.revanced.patcher.signature.MethodSignature
import app.revanced.patcher.signature.MethodSignatureMetadata
import app.revanced.patcher.signature.PatternScanMethod
import org.jf.dexlib2.AccessFlags
import org.jf.dexlib2.Opcode
import org.jf.dexlib2.iface.instruction.formats.Instruction11x
import org.jf.dexlib2.iface.instruction.formats.Instruction35c


private val packageMetadata = listOf(
    PackageMetadata(
        "com.google.android.youtube",
        listOf("17.03.38")
    ),
)

private val patchMetadata = PatchMetadata(
    "home-ads",
    "Home Ads Patch",
    "Patch to remove ads in YouTube",
    packageMetadata,
    "0.0.1"
)

private val signatureDescription = "Required signature for ${patchMetadata.name}. Discovered in version 17.03.38."

class HomeAdsPatch : Patch(
    patchMetadata,
    listOf(
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-1",
                MethodMetadata(
                    "Ljco;",
                    "k",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "Z",
            AccessFlags.PRIVATE or AccessFlags.FINAL,
            listOf("L"),
            listOf(
                Opcode.INVOKE_DIRECT,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.GOTO,
                Opcode.IGET_OBJECT,
                Opcode.IF_EQ,
                Opcode.CONST,
                Opcode.GOTO,
                Opcode.CONST,
                Opcode.INVOKE_DIRECT,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.INVOKE_DIRECT,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.INVOKE_STATIC,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.INVOKE_STATIC,
                Opcode.IGET_OBJECT,
                Opcode.GOTO,
                Opcode.IGET_OBJECT,
                Opcode.INVOKE_STATIC,
                Opcode.MOVE_RESULT,
                Opcode.IF_EQZ,
                Opcode.IGET_OBJECT,
                Opcode.IF_EQ,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-2",
                MethodMetadata(
                    "Ljlv;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "I"
            ),
            listOf(
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.INVOKE_DIRECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CONST,
                Opcode.CONST_4,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT,
                Opcode.IPUT_BOOLEAN,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-3",
                MethodMetadata(
                    "Ljkt;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "[B",
                "[B",
                "[B",
                "[B",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-4",
                MethodMetadata(
                    "Ljkx;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf("L", "I"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-5",
                MethodMetadata(
                    "Ljla;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf("L", "I"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-6",
                MethodMetadata(
                    "Ljld;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "[B",
                "[B",
                "[B",
                "[B",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-7",
                MethodMetadata(
                    "Ljle;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "I",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-8",
                MethodMetadata(
                    "Ljlh;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "[B",
                "[B",
                "[B",
                "[B",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-9",
                MethodMetadata(
                    "Ljli;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "[B",
                "[B",
                "[B",
                "[B",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.INVOKE_DIRECT,
                Opcode.IPUT_OBJECT,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-10",
                MethodMetadata(
                    "Ljlj;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf(
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "L",
                "[B",
                "[B",
                "[B",
                "[B",
                "[B"
            ),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_4,
                Opcode.INVOKE_DIRECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-11",
                MethodMetadata(
                    "Ljiw;",
                    "lG",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.FINAL or AccessFlags.BRIDGE or AccessFlags.SYNTHETIC,
            listOf("L", "L"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.INVOKE_STATIC,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-12",
                MethodMetadata(
                    "Ljjb;",
                    "b",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PRIVATE or AccessFlags.FINAL,
            listOf(),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.INVOKE_DIRECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST_4,
                Opcode.INVOKE_VIRTUAL,
                Opcode.CONST_4,
                Opcode.INVOKE_VIRTUAL,
                Opcode.RETURN_VOID
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-13",
                MethodMetadata(
                    "Ljjv;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf("L", "L", "L", "L", "Z", "L", "[B"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST_4,
                Opcode.INVOKE_STATIC,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-14",
                MethodMetadata(
                    "Ljjk;",
                    "b",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PRIVATE or AccessFlags.FINAL,
            listOf(),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.RETURN_VOID
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-15",
                MethodMetadata(
                    "Ljkl;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PUBLIC or AccessFlags.CONSTRUCTOR,
            listOf("L", "I"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IGET_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_4,
                Opcode.INVOKE_DIRECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_4,
                Opcode.INVOKE_DIRECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT,
                Opcode.IGET_BOOLEAN,
                Opcode.MOVE_FROM16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.INVOKE_DIRECT_RANGE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.IGET_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.IGET_BOOLEAN,
                Opcode.IGET_OBJECT,
                Opcode.IGET_OBJECT,
                Opcode.CONST_4,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.MOVE_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-16",
                MethodMetadata(
                    "Ljkp;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf("L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "Z", "L", "L", "[B", "[B", "[B", "[B", "[B"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_4,
                Opcode.INVOKE_DIRECT,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT_RANGE,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_16,
                Opcode.INVOKE_DIRECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.CONST_4,
                Opcode.INVOKE_DIRECT,
                Opcode.NEW_INSTANCE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.INVOKE_DIRECT_RANGE,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.IPUT_OBJECT,
                Opcode.NEW_INSTANCE,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.CONST_16,
                Opcode.MOVE_OBJECT_FROM16,
                Opcode.MOVE_OBJECT_FROM16,
            )
        ),
        MethodSignature(
            MethodSignatureMetadata(
                "home-ads-method-17",
                MethodMetadata(
                    "Ljkq;",
                    "<init>",
                ),
                PatternScanMethod.Fuzzy(2), // FIXME: Test this threshold and find the best value.
                packageMetadata,
                signatureDescription,
                "0.0.1"
            ),
            "V",
            AccessFlags.PROTECTED or AccessFlags.CONSTRUCTOR,
            listOf("L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "L", "[B", "[B", "[B", "[B", "[B"),
            listOf(
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
                Opcode.IPUT_OBJECT,
                Opcode.CONST,
                Opcode.INVOKE_VIRTUAL,
                Opcode.MOVE_RESULT_OBJECT,
                Opcode.MOVE_OBJECT,
                Opcode.CHECK_CAST,
            )
        )
    )
) {
    override fun execute(patcherData: PatcherData): PatchResult {
        for (i in 0 until signatures.count()) {
            val signature = signatures.elementAt(i)
            val result = signature.result!!
            val implementation = result.method.implementation!!
            val index = result.scanData.startIndex
            val instructions = implementation.instructions
            val register =
                if (i < 2)
                    (instructions[index - 1] as Instruction11x).registerA
                else
                    (instructions[index] as Instruction35c).registerC

            implementation.injectHideCall(index, register)
        }

        return PatchResultSuccess()
    }
}